from shutil import copyfile
from datetime import datetime

import sys
import os
import chevron
import zipfile


class ModuleEngine:
    def __init__(self, name: str):
        """
        Engine for generating or testing a module.

        :param name:
        """
        if not isinstance(name, str):
            raise ValueError("module name must be of type string")

        self._path = f'{os.getcwd()}/{name.lower()}'
        self._pkg = sys.modules['vanirio'].__path__[0]
        self._name = name
        self._timestamp = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")

    def create(self, example: bool = False):
        """
        When no directory exist for the defined path,
        create the directory and return True. Otherwise
        skip the ModuleEngine execution to avoid possible
        overwrite of module files.

        :return:
        """
        if not os.path.isdir(self._path):
            os.mkdir(self._path, 0o777)

            _init_src = f'{self._pkg}/engine/templates/__init__.py'
            _init_dst = f'{self._path}/__init__.py'
            copyfile(_init_src, _init_dst)

            self.__runtime(example)
            self.__testing(example)
            self.__commissioning(example)

    def __runtime(self, example: bool = False):
        """
        Render the runtime file.

        :return:
        """
        _runtime = f'{self._pkg}/engine/templates/runtime.mu'
        with open(_runtime, 'r') as f:
            content = chevron.render(f, {
                'name': self._name,
                'timestamp': self._timestamp,
                'example': example,
                'not_example': not example
            })

            _com = f'{self._path}/runtime.py'
            with open(_com, 'w') as fw:
                fw.write(content)

    def __commissioning(self, example: bool = False):
        """
        Render the commissioning file.

        :return:
        """
        _runtime = f'{self._pkg}/engine/templates/commissioning.mu'
        with open(_runtime, 'r') as f:
            content = chevron.render(f, {
                'name': self._name,
                'timestamp': self._timestamp,
                'example': example,
            })

            _com = f'{self._path}/commissioning.py'
            with open(_com, 'w') as fw:
                fw.write(content)

    def __testing(self, example: bool = False):
        """
        Render the testing file.

        :return:
        """
        _runtime = f'{self._pkg}/engine/templates/testing.mu'
        with open(_runtime, 'r') as f:
            content = chevron.render(f, {
                'name': self._name,
                'timestamp': self._timestamp,
                'example': example,
                'not_example': not example
            })

            _com = f'{self._path}/testing.py'
            with open(_com, 'w') as fw:
                fw.write(content)

    def test(self) -> bool:
        """
        Execute the module with the test information
        provided by the user in the module directory.

        :return: successful?
        """
        return False

    def package(self):
        """
        To help an engineer, this method will select the module files and
        attach the files to a zip file which gets the same name as the module
        name. Afterwards, the engineer can easily upload the module to the cloud.
        The package method will not copy other existing files!

        :return:
        """
        if os.path.isdir(self._path):

            def zip_files(files: list):
                """
                Append the list of files to a zip named after the module.

                :param files:
                :return:
                """
                zf = zipfile.ZipFile(f'{self._path}/{self._name}.zip', 'w', zipfile.ZIP_DEFLATED)
                for path in files:
                    filename = os.path.basename(os.path.normpath(path))
                    zf.write(path, filename)
                zf.close()

            zip_files([
                f'{self._path}/__init__.py',
                f'{self._path}/runtime.py',
                f'{self._path}/commissioning.py',
                f'{self._path}/testing.py'
            ])
